@@ -12,27 +12,30 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 /*
  * Call Map View
  */
 
-#include <qwhatsthis.h>
-#include <qpopupmenu.h>
+
+#include "callmapview.h"
+
+#include <QPixmap>
+#include <Qt3Support/Q3PopupMenu>
 
 #include <klocale.h>
 #include <kiconloader.h>
-#include <kapplication.h>
 #include <kconfig.h>
+#include <kconfiggroup.h>
 
-#include "callmapview.h"
 #include "configuration.h"
 #include "listutils.h"
 #include "toplevel.h"
 
+
 //
 // CallMapView
 //
@@ -52,11 +55,12 @@
 
 CallMapView::CallMapView(bool showCallers, TraceItemView* parentView,
                          QWidget* parent, const char* name)
-  : TreeMapWidget(new CallMapBaseItem(), parent, name),  TraceItemView(parentView)
+  : TreeMapWidget(new CallMapBaseItem(), parent),  TraceItemView(parentView)
 {
+  setObjectName(name);
   _showCallers = showCallers;
 
-  setFieldType(0, i18n( "Name" ));
+  setFieldType(0, i18nc("A thing's name", "Name" ));
   setFieldType(1, i18n( "Cost" ));
   setFieldType(2, i18n( "Location" ));
   setFieldPosition(2, TreeMapItem::TopLeft);
@@ -89,7 +93,7 @@ CallMapView::CallMapView(bool showCallers, TraceItemView* parentView,
           SIGNAL(contextMenuRequested(TreeMapItem*,const QPoint &)),
           SLOT(context(TreeMapItem*,const QPoint &)));
 
-  QWhatsThis::add( this, whatsThis());
+  this->setWhatsThis( whatsThis());
 }
 
 QString CallMapView::whatsThis() const
@@ -143,14 +147,14 @@ void CallMapView::context(TreeMapItem* i,const QPoint & p)
 {
   if (!i) return;
 
-  QPopupMenu popup;
-  QPopupMenu fpopup; // select function subpopup
-  QPopupMenu vpopup; // visualisation subpopup
-  QPopupMenu dpopup; // split direction
-  QPopupMenu bpopup; // border subpopup
-  QPopupMenu l1popup; // depth limit subpopup
-  QPopupMenu l2popup; // function limit subpopup
-  QPopupMenu l3popup; // area limit subpopup
+  Q3PopupMenu popup;
+  Q3PopupMenu fpopup; // select function subpopup
+  Q3PopupMenu vpopup; // visualization subpopup
+  Q3PopupMenu dpopup; // split direction
+  Q3PopupMenu bpopup; // border subpopup
+  Q3PopupMenu l1popup; // depth limit subpopup
+  Q3PopupMenu l2popup; // function limit subpopup
+  Q3PopupMenu l3popup; // area limit subpopup
 
   TreeMapItem* item = i;
   int count;
@@ -168,9 +172,7 @@ void CallMapView::context(TreeMapItem* i,const QPoint & p)
     count = 0;
     while (count<Configuration::maxSymbolCount() && item) {
       QString name = item->text(0);
-      if ((int)name.length()>Configuration::maxSymbolLength())
-        name = name.left(Configuration::maxSymbolLength()) + "...";
-      fpopup.insertItem(name, 101+count);
+      fpopup.insertItem(Configuration::shortenSymbol(name), 101+count);
       item = item->parent();
       count++;
     }
@@ -195,14 +197,14 @@ void CallMapView::context(TreeMapItem* i,const QPoint & p)
   l1popup.setItemChecked(53, maxDepth==20);
   if (i) {
     l1popup.insertSeparator();
-    l1popup.insertItem(i18n("Depth of '%1' (%2)")
-                       .arg(shortCurrentName).arg(i->depth()), 55);
+    l1popup.insertItem(i18n("Depth of '%1' (%2)",
+                        shortCurrentName, i->depth()), 55);
     l1popup.setItemChecked(55, maxDepth == i->depth());
   }
   if (maxDepth>0) {
     l1popup.insertSeparator();
-    l1popup.insertItem(i18n("Decrement Depth (to %1)").arg(maxDepth-1), 56);
-    l1popup.insertItem(i18n("Increment Depth (to %1)").arg(maxDepth+1), 57);
+    l1popup.insertItem(i18n("Decrement Depth (to %1)", maxDepth-1), 56);
+    l1popup.insertItem(i18n("Increment Depth (to %1)", maxDepth+1), 57);
   }
 
   l2popup.setCheckable(true);
@@ -255,24 +257,24 @@ void CallMapView::context(TreeMapItem* i,const QPoint & p)
   if (i) {
     currentArea = i->width() * i->height();
     l3popup.insertSeparator();
-    l3popup.insertItem(i18n("Area of '%1' (%2)")
-                       .arg(shortCurrentName).arg(currentArea), 67);
+    l3popup.insertItem(i18n("Area of '%1' (%2)",
+                        shortCurrentName, currentArea), 67);
     l3popup.setItemChecked(67, mArea == currentArea);
   }
   if (mArea>0) {
     l3popup.insertSeparator();
-    l3popup.insertItem(i18n("Double Area Limit (to %1)")
-                       .arg(mArea*2), 68);
-    l3popup.insertItem(i18n("Half Area Limit (to %1)")
-                       .arg(mArea/2), 69);
+    l3popup.insertItem(i18n("Double Area Limit (to %1)",
+                        mArea*2), 68);
+    l3popup.insertItem(i18n("Half Area Limit (to %1)",
+                        mArea/2), 69);
   }
 
   popup.insertSeparator();
 
   vpopup.setCheckable(true);
-  popup.insertItem(i18n("Visualisation"), &vpopup, 10);
+  popup.insertItem(i18n("Visualization"), &vpopup, 10);
 
-  QPopupMenu splitpopup;
+  Q3PopupMenu splitpopup;
   addSplitDirectionItems(&splitpopup, 1001);
   vpopup.insertItem(i18n("Split Direction"), &splitpopup, 1000);
 
@@ -345,35 +347,38 @@ void CallMapView::context(TreeMapItem* i,const QPoint & p)
     return;
   }
 
+  bool newSetting;
+
   switch(r) {
   case 20:
-    setFieldVisible(0, !vpopup.isItemChecked(20));
+    setFieldVisible(0, !fieldVisible(0));
     break;
 
   case 21:
-    setFieldVisible(1, !vpopup.isItemChecked(21));
+    setFieldVisible(1, !fieldVisible(1));
     break;
 
   case 22:
-    setFieldVisible(2, !vpopup.isItemChecked(22));
+    setFieldVisible(2, !fieldVisible(2));
     break;
 
   case 23:
-    setFieldVisible(3, !vpopup.isItemChecked(23));
+    setFieldVisible(3, !fieldVisible(3));
     break;
 
   case 24:
-    setFieldForced(0, !vpopup.isItemChecked(24));
-    setFieldForced(1, !vpopup.isItemChecked(24));
-    setFieldForced(2, !vpopup.isItemChecked(24));
-    setFieldForced(3, !vpopup.isItemChecked(24));
+    newSetting = !fieldForced(0);
+    setFieldForced(0, newSetting);
+    setFieldForced(1, newSetting);
+    setFieldForced(2, newSetting);
+    setFieldForced(3, newSetting);
     break;
 
-  case 25: setAllowRotation(!vpopup.isItemChecked(25)); break;
-  case 26: setShadingEnabled(!vpopup.isItemChecked(26)); break;
+  case 25: setAllowRotation(!allowRotation()); break;
+  case 26: setShadingEnabled(!isShadingEnabled()); break;
 
   case 40:
-    setSkipIncorrectBorder(!vpopup.isItemChecked(40));
+    setSkipIncorrectBorder(!skipIncorrectBorder());
     break;
 
   case 42: setBorderWidth(0); break;
@@ -389,7 +394,7 @@ void CallMapView::context(TreeMapItem* i,const QPoint & p)
   case 56: setMaxDrawingDepth(maxDepth-1); break;
   case 57: setMaxDrawingDepth(maxDepth+1); break;
 
-  case 200: setFieldStop(0, QString::null); break;
+  case 200: setFieldStop(0, QString()); break;
 
   case 60: setMinimalArea(-1); break;
   case 61: setMinimalArea(10); break;
@@ -428,7 +433,7 @@ void CallMapView::selectedSlot(TreeMapItem* item, bool kbd)
   if (item->text(0).isEmpty()) return;
 
   if (kbd) {
-      QString msg = i18n("Call Map: Current is '%1'").arg(item->text(0));
+      QString msg = i18n("Call Map: Current is '%1'", item->text(0));
       if (_topLevel)
 	  _topLevel->showMessage(msg, 5000);
   }
@@ -470,7 +475,7 @@ TraceItem* CallMapView::canShow(TraceItem* i)
 
 void CallMapView::doUpdate(int changeType)
 {
-    if (changeType == costType2Changed) return;
+    if (changeType == eventType2Changed) return;
 
     // if there is a selected item, always draw marking...
     if (changeType & selectedItemChanged) {
@@ -517,7 +522,7 @@ void CallMapView::doUpdate(int changeType)
     base()->refresh();
   }
   else if ((changeType & partsChanged) ||
-           (changeType & costTypeChanged)) {
+           (changeType & eventTypeChanged)) {
     /* we need to do the draw order sorting again as the values change */
     resort();
     redraw();
@@ -531,7 +536,7 @@ void CallMapView::doUpdate(int changeType)
 QColor CallMapView::groupColor(TraceFunction* f) const
 {
   if (!f)
-    return colorGroup().button();
+      return palette().color( QPalette::Button );
 
   return Configuration::functionColor(_groupType, f);
 }
@@ -551,9 +556,9 @@ QString CallMapView::tipString(TreeMapItem* i) const
       itemTip = itemTip.left(Configuration::maxSymbolLength()) + "...";
 
     if (!i->text(1).isEmpty())
-      itemTip += " (" + i->text(1) + ")";
+      itemTip += " (" + i->text(1) + ')';
 
-    if (!tip.isEmpty()) tip += "\n";
+    if (!tip.isEmpty()) tip += '\n';
 
     tip += itemTip;
     i = i->parent();
@@ -570,7 +575,7 @@ TraceCost* CallMapView::totalCost()
   TraceFunction* f = ((CallMapBaseItem*)base())->function();
   if (!f) return 0;
 
-  return Configuration::showExpanded() ? f->cumulative() : f->data();
+  return Configuration::showExpanded() ? f->inclusive() : f->data();
 }
 
 
@@ -601,13 +606,13 @@ QString CallMapBaseItem::text(int textNo) const
     return _f->prettyName();
   }
 
-  if (!_f) return QString::null;
+  if (!_f) return QString();
 
-  if (textNo == 2) return _f->location();
+  if (textNo == 2) return _f->prettyLocation();
   if (textNo == 3) return _f->calledCount().pretty();
-  if (textNo != 1) return QString::null;
+  if (textNo != 1) return QString();
 
-  TraceCostType* ct = ((CallMapView*)widget())->costType();
+  TraceEventType* ct = ((CallMapView*)widget())->eventType();
   TraceCost* t      = ((CallMapView*)widget())->totalCost();
 
   if (Configuration::showPercentage()) {
@@ -615,23 +620,23 @@ QString CallMapBaseItem::text(int textNo) const
       if (total == 0.0)
 	  sum = 100.0;
       else
-	  sum = 100.0 * _f->cumulative()->subCost(ct) / total;
+	  sum = 100.0 * _f->inclusive()->subCost(ct) / total;
 
       return QString("%1 %")
 	  .arg(sum, 0, 'f', Configuration::percentPrecision());
   }
-  return _f->cumulative()->prettySubCost(ct);
+  return _f->inclusive()->prettySubCost(ct);
 }
 
 QPixmap CallMapBaseItem::pixmap(int i) const
 {
     if ((i != 1) || !_f) return QPixmap();
 
-    TraceCostType* ct = ((CallMapView*)widget())->costType();
+    TraceEventType* ct = ((CallMapView*)widget())->eventType();
     TraceCost* t      = ((CallMapView*)widget())->totalCost();
 
     // colored level meter with frame
-    return costPixmap( ct, _f->cumulative(), (double) (t->subCost(ct)), true);
+    return costPixmap( ct, _f->inclusive(), (double) (t->subCost(ct)), true);
 }
 
 
@@ -639,9 +644,9 @@ double CallMapBaseItem::value() const
 {
   if (!_f) return 0.0;
 
-  TraceCostType* ct;
-  ct = ((CallMapView*)widget())->costType();
-  return (double) _f->cumulative()->subCost(ct);
+  TraceEventType* ct;
+  ct = ((CallMapView*)widget())->eventType();
+  return (double) _f->inclusive()->subCost(ct);
 }
 
 
@@ -654,7 +659,7 @@ double CallMapBaseItem::sum() const
   if (w->showCallers())
     return 0.0;
   else
-    return (double) _f->cumulative()->subCost(w->costType());
+    return (double) _f->inclusive()->subCost(w->eventType());
 }
 
 
@@ -679,7 +684,7 @@ TreeMapItemList* CallMapBaseItem::children()
       TraceCallList l = _f->callers();
       for (call=l.first();call;call=l.next()) {
 
-        // don't show calls inside of a cycle
+        // do not show calls inside of a cycle
         if (call->inCycle()>0) continue;
         if (call->isRecursion()) continue;
 
@@ -692,7 +697,7 @@ TreeMapItemList* CallMapBaseItem::children()
       TraceCallList l = _f->callings();
       for (call=l.first();call;call=l.next()) {
 
-        // don't show calls inside of a cycle
+        // do not show calls inside of a cycle
         if (call->inCycle()>0) continue;
         if (call->isRecursion()) continue;
 
@@ -701,7 +706,7 @@ TreeMapItemList* CallMapBaseItem::children()
         addItem(i);
       }
 
-      setSum(_f->cumulative()->subCost(w->costType()));
+      setSum(_f->inclusive()->subCost(w->eventType()));
     }
     setSorting(-2, false);
   }
@@ -743,12 +748,12 @@ QString CallMapCallingItem::text(int textNo) const
     return _c->calledName();
   }
 
-  if (textNo == 2) return _c->called()->location();
+  if (textNo == 2) return _c->called()->prettyLocation();
   if (textNo == 3) return SubCost(_factor * _c->callCount()).pretty();
-  if (textNo != 1) return QString::null;
+  if (textNo != 1) return QString();
 
-  TraceCostType* ct;
-  ct = ((CallMapView*)widget())->costType();
+  TraceEventType* ct;
+  ct = ((CallMapView*)widget())->eventType();
 
   SubCost val = SubCost(_factor * _c->subCost(ct));
   if (Configuration::showPercentage()) {
@@ -766,7 +771,7 @@ QPixmap CallMapCallingItem::pixmap(int i) const
     if (i != 1) return QPixmap();
 
     // Cost pixmap
-    TraceCostType* ct = ((CallMapView*)widget())->costType();
+    TraceEventType* ct = ((CallMapView*)widget())->eventType();
     TraceCost* t      = ((CallMapView*)widget())->totalCost();
 
     // colored level meter with frame
@@ -776,8 +781,8 @@ QPixmap CallMapCallingItem::pixmap(int i) const
 
 double CallMapCallingItem::value() const
 {
-  TraceCostType* ct;
-  ct = ((CallMapView*)widget())->costType();
+  TraceEventType* ct;
+  ct = ((CallMapView*)widget())->eventType();
   return _factor * _c->subCost(ct);
 }
 
@@ -797,11 +802,11 @@ TreeMapItemList* CallMapCallingItem::children()
   if (!initialized()) {
     if (0) qDebug("Create Calling subitems (%s)", path(0).join("/").ascii());
 
-    TraceCostType* ct;
-    ct = ((CallMapView*)widget())->costType();
+    TraceEventType* ct;
+    ct = ((CallMapView*)widget())->eventType();
 
     // same as sum()
-    SubCost s = _c->called()->cumulative()->subCost(ct);
+    SubCost s = _c->called()->inclusive()->subCost(ct);
     SubCost v = _c->subCost(ct);
     if (v>s) {
       qDebug("Warning: CallingItem subVal %u > Sum %u (%s)",
@@ -821,7 +826,7 @@ TreeMapItemList* CallMapCallingItem::children()
     TraceCallList l = _c->called()->callings();
     for (call=l.first();call;call=l.next()) {
 
-      // don't show calls inside of a cycle
+      // do not show calls inside of a cycle
       if (call->inCycle()>0) continue;
       if (call->isRecursion()) continue;
 
@@ -860,12 +865,12 @@ QString CallMapCallerItem::text(int textNo) const
     return _c->callerName();
   }
 
-  if (textNo == 2) return _c->caller()->location();
+  if (textNo == 2) return _c->caller()->prettyLocation();
   if (textNo == 3) return SubCost(_factor * _c->callCount()).pretty();
-  if (textNo != 1) return QString::null;
+  if (textNo != 1) return QString();
 
-  TraceCostType* ct;
-  ct = ((CallMapView*)widget())->costType();
+  TraceEventType* ct;
+  ct = ((CallMapView*)widget())->eventType();
 
   SubCost val = SubCost(_factor * _c->subCost(ct));
   if (Configuration::showPercentage()) {
@@ -883,7 +888,7 @@ QPixmap CallMapCallerItem::pixmap(int i) const
     if (i != 1) return QPixmap();
 
     // Cost pixmap
-    TraceCostType* ct = ((CallMapView*)widget())->costType();
+    TraceEventType* ct = ((CallMapView*)widget())->eventType();
     TraceCost* t      = ((CallMapView*)widget())->totalCost();
 
     // colored level meter with frame
@@ -893,8 +898,8 @@ QPixmap CallMapCallerItem::pixmap(int i) const
 
 double CallMapCallerItem::value() const
 {
-  TraceCostType* ct;
-  ct = ((CallMapView*)widget())->costType();
+  TraceEventType* ct;
+  ct = ((CallMapView*)widget())->eventType();
   return (double) _c->subCost(ct);
 }
 
@@ -909,10 +914,10 @@ TreeMapItemList* CallMapCallerItem::children()
   if (!initialized()) {
     //qDebug("Create Caller subitems (%s)", name().ascii());
 
-    TraceCostType* ct;
-    ct = ((CallMapView*)widget())->costType();
+    TraceEventType* ct;
+    ct = ((CallMapView*)widget())->eventType();
 
-    SubCost s = _c->caller()->cumulative()->subCost(ct);
+    SubCost s = _c->caller()->inclusive()->subCost(ct);
     SubCost v = _c->subCost(ct);
     double newFactor = _factor * v / s;
 
@@ -929,7 +934,7 @@ TreeMapItemList* CallMapCallerItem::children()
     TraceCallList l = _c->caller()->callers();
     for (call=l.first();call;call=l.next()) {
 
-      // don't show calls inside of a cycle
+      // do not show calls inside of a cycle
       if (call->inCycle()>0) continue;
       if (call->isRecursion()) continue;
 
@@ -949,51 +954,50 @@ QColor CallMapCallerItem::backColor() const
 }
 
 void CallMapView::readViewConfig(KConfig* c,
-				 QString prefix, QString postfix, bool)
+                                 const QString& prefix, const QString& postfix, bool)
 {
-    KConfigGroup* g = configGroup(c, prefix, postfix);
+    KConfigGroup g = configGroup(c, prefix, postfix);
 
-    setSplitMode(g->readEntry("SplitMode", DEFAULT_SPLITMODE));
+    setSplitMode(g.readEntry("SplitMode", DEFAULT_SPLITMODE));
 
-    setFieldVisible(0, g->readBoolEntry("DrawName", DEFAULT_DRAWNAME));
-    setFieldVisible(1, g->readBoolEntry("DrawCost", DEFAULT_DRAWCOST));
-    setFieldVisible(2, g->readBoolEntry("DrawLocation", DEFAULT_DRAWLOCATION));
-    setFieldVisible(3, g->readBoolEntry("DrawCalls", DEFAULT_DRAWCALLS));
+    setFieldVisible(0, g.readEntry("DrawName", DEFAULT_DRAWNAME));
+    setFieldVisible(1, g.readEntry("DrawCost", DEFAULT_DRAWCOST));
+    setFieldVisible(2, g.readEntry("DrawLocation", DEFAULT_DRAWLOCATION));
+    setFieldVisible(3, g.readEntry("DrawCalls", DEFAULT_DRAWCALLS));
 
-    bool enable = g->readBoolEntry("ForceStrings", DEFAULT_FORCESTRINGS);
+    bool enable = g.readEntry("ForceStrings", DEFAULT_FORCESTRINGS);
     setFieldForced(0, enable);
     setFieldForced(1, enable);
     setFieldForced(2, enable);
     setFieldForced(3, enable);
 
-    setAllowRotation(g->readBoolEntry("AllowRotation", DEFAULT_ROTATION));
-    setShadingEnabled(g->readBoolEntry("Shading", DEFAULT_SHADING));
-    setFieldStop(0, g->readEntry("StopName"));
-    setMaxDrawingDepth(g->readNumEntry("MaxDepth", -1));
-    setMinimalArea(g->readNumEntry("MaxArea", DEFAULT_MAXAREA));
+    setAllowRotation(g.readEntry("AllowRotation", DEFAULT_ROTATION));
+    setShadingEnabled(g.readEntry("Shading", DEFAULT_SHADING));
+    setFieldStop(0, g.readEntry("StopName"));
+    setMaxDrawingDepth(g.readEntry("MaxDepth", -1));
+    setMinimalArea(g.readEntry("MaxArea", DEFAULT_MAXAREA));
 
-    delete g;
 }
 
-void CallMapView::saveViewConfig(KConfig* c, 
-				 QString prefix, QString postfix, bool)
+void CallMapView::saveViewConfig(KConfig* c,
+                                 const QString& prefix, const QString& postfix, bool)
 {
     KConfigGroup g(c, (prefix+postfix).ascii());
 
-    writeConfigEntry(&g, "SplitMode", splitModeString(), DEFAULT_SPLITMODE);
-    writeConfigEntry(&g, "DrawName", fieldVisible(0), DEFAULT_DRAWNAME);
-    writeConfigEntry(&g, "DrawCost", fieldVisible(1), DEFAULT_DRAWCOST);
-    writeConfigEntry(&g, "DrawLocation", fieldVisible(2), DEFAULT_DRAWLOCATION);
-    writeConfigEntry(&g, "DrawCalls", fieldVisible(3), DEFAULT_DRAWCALLS);
+    writeConfigEntry(g, "SplitMode", splitModeString(), DEFAULT_SPLITMODE);
+    writeConfigEntry(g, "DrawName", fieldVisible(0), DEFAULT_DRAWNAME);
+    writeConfigEntry(g, "DrawCost", fieldVisible(1), DEFAULT_DRAWCOST);
+    writeConfigEntry(g, "DrawLocation", fieldVisible(2), DEFAULT_DRAWLOCATION);
+    writeConfigEntry(g, "DrawCalls", fieldVisible(3), DEFAULT_DRAWCALLS);
     // when option for all text (0-3)
-    writeConfigEntry(&g, "ForceStrings", fieldForced(0), DEFAULT_FORCESTRINGS);
+    writeConfigEntry(g, "ForceStrings", fieldForced(0), DEFAULT_FORCESTRINGS);
 
-    writeConfigEntry(&g, "AllowRotation", allowRotation(), DEFAULT_ROTATION);
-    writeConfigEntry(&g, "Shading", isShadingEnabled(), DEFAULT_SHADING);
+    writeConfigEntry(g, "AllowRotation", allowRotation(), DEFAULT_ROTATION);
+    writeConfigEntry(g, "Shading", isShadingEnabled(), DEFAULT_SHADING);
 
-    writeConfigEntry(&g, "StopName", fieldStop(0), "");
-    writeConfigEntry(&g, "MaxDepth", maxDrawingDepth(), -1);
-    writeConfigEntry(&g, "MaxArea", minimalArea(), DEFAULT_MAXAREA);
+    writeConfigEntry(g, "StopName", fieldStop(0), "");
+    writeConfigEntry(g, "MaxDepth", maxDrawingDepth(), -1);
+    writeConfigEntry(g, "MaxArea", minimalArea(), DEFAULT_MAXAREA);
 }
 
 #include "callmapview.moc"
