@@ -15,6 +15,7 @@
 // qt/kde includes
 #include <kdebug.h>
 #include <typeinfo>
+#include <kmessagebox.h>
 
 // local includes
 #include "uml.h"
@@ -25,6 +26,7 @@
 #include "associationwidget.h"
 #include "statewidget.h"
 #include "activitywidget.h"
+#include "signalwidget.h"
 #include "forkjoinwidget.h"
 
 
@@ -40,7 +42,7 @@ bool allowAssociation( Association_Type/* assocType*/, const std::type_info/* &t
 
 }
 
-bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widget ) {
+bool AssocRules::allowAssociation( Uml::Association_Type assocType, UMLWidget * widget ) {
     Widget_Type widgetType = widget -> getBaseType();
     bool bValid = false;
     for (int i = 0; i < m_nNumRules; i++) {
@@ -61,8 +63,7 @@ bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widge
             return false;
     }
     AssociationWidgetList list = widget -> getAssocList();
-    AssociationWidgetListIt it( list );
-    AssociationWidget * assoc = 0;
+
     switch( assocType ) {
     case at_Association:
     case at_UniAssociation:
@@ -81,10 +82,9 @@ bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widge
         break;
 
     case at_Realization:  // one connected to widget only (a or b)
-        while( ( assoc = it.current() ) ) {
+        foreach ( AssociationWidget* assoc, list ) {
             if( assoc -> getAssocType() == at_Realization )
                 return false;
-            ++it;
         }
         return true;
         break;
@@ -97,6 +97,7 @@ bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widge
         break;
 
     case at_Activity:
+    case at_Exception:
         {
             ActivityWidget *pActivity = dynamic_cast<ActivityWidget*>(widget);
             return (pActivity == NULL || pActivity->getActivityType() != ActivityWidget::End);
@@ -106,8 +107,19 @@ bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widge
     case at_Anchor:
         return true;
         break;
+
+    case at_Category2Parent:
+        if ( widgetType == wt_Category )
+            return true;
+        break;
+
+    case at_Child2Category:
+        if ( widgetType == wt_Entity )
+            return true;
+        break;
+
     default:
-        kWarning() << "allowAssociation() on unknown type" << endl;
+        uWarning() << "allowAssociation() on unknown type";
         break;
     }
     return false;
@@ -115,12 +127,17 @@ bool AssocRules::allowAssociation( Association_Type assocType, UMLWidget * widge
 
 // when we know what we are going to connect both ends of the association to, we can
 // use this method.
-bool AssocRules::allowAssociation( Association_Type assocType,
+bool AssocRules::allowAssociation( Uml::Association_Type assocType,
                                    UMLWidget * widgetA, UMLWidget * widgetB,
                                    bool extendedCheck ) {
     Widget_Type widgetTypeA = widgetA->getBaseType();
     Widget_Type widgetTypeB = widgetB->getBaseType();
     bool bValid = false;
+
+    if ( widgetA->getUMLObject() == widgetB->getUMLObject() ) {
+        return allowSelf( assocType, widgetTypeA );
+    }
+
     for (int i = 0; i < m_nNumRules; i++) {
         if (assocType == m_AssocRules[i].assoc_type) {
             if( (widgetTypeA == m_AssocRules[i].widgetA_type &&
@@ -139,12 +156,16 @@ bool AssocRules::allowAssociation( Association_Type assocType,
     if (!bValid) {
         return false;
     }
+
     AssociationWidgetList list = widgetB -> getAssocList();
-    AssociationWidgetListIt it( list );
-    AssociationWidget * assoc = 0;
+
     switch( assocType ) {
-    case at_Association:
     case at_Association_Self:
+        if ( widgetA->getUMLObject() == widgetB->getUMLObject() )
+            return true;
+        break;
+
+    case at_Association:
     case at_UniAssociation:
     case at_Dependency:
     case at_Coll_Message:
@@ -156,22 +177,20 @@ bool AssocRules::allowAssociation( Association_Type assocType,
     case at_Composition:   // can't have mutual composition
     case at_Containment:   // can't have mutual containment
     case at_Generalization://can have many sub/super types but can't sup/sub each
-        while( ( assoc = it.current() ) ) {
+        foreach ( AssociationWidget * assoc, list ) {
             if( ( widgetA == assoc -> getWidget(A) || widgetA == assoc -> getWidget(B) )
                     && assoc->getAssocType() == assocType )
                 return false;
-            ++it;
         }
         return true;
         break;
 
     case at_Realization: // can only connect to abstract (interface) classes
-        while( ( assoc = it.current() ) ) {
+        foreach( AssociationWidget * assoc, list ) {
             if( ( widgetA == assoc->getWidget(A) || widgetA == assoc->getWidget(B) )
                     && assoc->getAssocType() == at_Realization ) {
                 return false;
             }
-            ++it;
         }
         if (widgetB->getBaseType() == wt_Class) {
             return widgetB->getUMLObject()->getAbstract();
@@ -197,30 +216,42 @@ bool AssocRules::allowAssociation( Association_Type assocType,
         break;
 
     case at_Activity:
+    case at_Exception:
         {
+
             ActivityWidget *actA = dynamic_cast<ActivityWidget*>(widgetA);
             ActivityWidget *actB = dynamic_cast<ActivityWidget*>(widgetB);
+
+            bool isSignal = false;
+            bool isObjectNode = false;
+
+            if (widgetTypeA == wt_Signal)
+                isSignal = true;
+            else if (widgetTypeA == wt_ObjectNode)
+                isObjectNode = true;
+
             // no transitions to initial activity allowed
-            if (actB && actB->getActivityType() == ActivityWidget::Initial)
+            if (actB && actB->getActivityType() == ActivityWidget::Initial) {
                 return false;
-            // Fork_DEPRECATED here means "not applicable".
-            ActivityWidget::ActivityType actTypeA = ActivityWidget::Fork_DEPRECATED;
+            }
+            // actType -1 here means "not applicable".
+            int actTypeA = -1;
             if (actA)
                 actTypeA = actA->getActivityType();
-            ActivityWidget::ActivityType actTypeB = ActivityWidget::Fork_DEPRECATED;
+            int actTypeB = -1;
             if (actB)
                 actTypeB = actB->getActivityType();
-            // only from a normal, branch or fork activity to the end
-            if (actTypeB == ActivityWidget::End &&
+            // only from a signalwidget a objectnode widget, a normal activity, branch or fork activity, to the end
+            if ((actTypeB == ActivityWidget::End || actTypeB == ActivityWidget::Final) &&
                 actTypeA != ActivityWidget::Normal &&
                 actTypeA != ActivityWidget::Branch &&
-                dynamic_cast<ForkJoinWidget*>(widgetA) == NULL) {
+                dynamic_cast<ForkJoinWidget*>(widgetA) == NULL && !isSignal &&!isObjectNode) {
                 return false;
             }
             // only Forks and Branches can have more than one "outgoing" transition
             if (actA != NULL && actTypeA != ActivityWidget::Branch) {
                 AssociationWidgetList list = widgetA->getAssocList();
-                for (AssociationWidget* assoc = list.first(); assoc; assoc = list.next()) {
+                foreach (AssociationWidget* assoc , list ) {
                     if (assoc->getWidget(A) == widgetA) {
                         return false;
                     }
@@ -234,21 +265,33 @@ bool AssocRules::allowAssociation( Association_Type assocType,
         return true;
         break;
 
+    case at_Category2Parent:
+        if ( widgetTypeA == wt_Category && widgetTypeB == wt_Entity ) {
+            return true;
+        }
+        break;
+
+    case at_Child2Category:
+        if ( widgetTypeA == wt_Entity && widgetTypeB == wt_Category ) {
+            return true;
+        }
+        break;
+
     default:
-        kWarning() << "allowAssociation() on unknown type" << endl;
+        uWarning() << "allowAssociation() on unknown type";
         break;
     }
     return false;
 }
 
-bool AssocRules::allowRole( Association_Type assocType ) {
+bool AssocRules::allowRole( Uml::Association_Type assocType ) {
     for( int i = 0; i < m_nNumRules; i++ )
         if( assocType == m_AssocRules[ i ].assoc_type )
             return m_AssocRules[ i ].role;
     return false;
 }
 
-bool AssocRules::allowMultiplicity( Association_Type assocType, Widget_Type widgetType ) {
+bool AssocRules::allowMultiplicity( Uml::Association_Type assocType, Uml::Widget_Type widgetType ) {
     for( int i = 0; i < m_nNumRules; i++ )
         if( assocType == m_AssocRules[ i ].assoc_type )
             if( widgetType == m_AssocRules[ i ].widgetA_type || widgetType == m_AssocRules[ i ].widgetB_type )
@@ -256,7 +299,7 @@ bool AssocRules::allowMultiplicity( Association_Type assocType, Widget_Type widg
     return false;
 }
 
-bool AssocRules::allowSelf( Association_Type assocType, Widget_Type widgetType ) {
+bool AssocRules::allowSelf( Uml::Association_Type assocType, Uml::Widget_Type widgetType ) {
     for( int i = 0; i < m_nNumRules; i++ )
         if( assocType == m_AssocRules[ i ].assoc_type )
             if( widgetType == m_AssocRules[ i ].widgetA_type || widgetType == m_AssocRules[ i ].widgetB_type )
@@ -265,7 +308,7 @@ bool AssocRules::allowSelf( Association_Type assocType, Widget_Type widgetType )
     return false;
 }
 
-Association_Type AssocRules::isGeneralisationOrRealisation(UMLWidget* widgetA, UMLWidget* widgetB) {
+Uml::Association_Type AssocRules::isGeneralisationOrRealisation(UMLWidget* widgetA, UMLWidget* widgetB) {
     Widget_Type widgetTypeA = widgetA->getBaseType();
     Widget_Type widgetTypeB = widgetB->getBaseType();
     for (int i = 0; i < m_nNumRules; i++) {
@@ -365,9 +408,22 @@ AssocRules::Assoc_Rule AssocRules::m_AssocRules []= {
     { at_State,         wt_State,       wt_State,       true,   false,  true,   true  },
     { at_State,         wt_ForkJoin,    wt_State,       true,   false,  true,   true  },
     { at_State,         wt_State,       wt_ForkJoin,    true,   false,  true,   true  },
+    { at_Activity,      wt_Signal,      wt_Activity,    true,   false,  true,   true  },
+    { at_Activity,      wt_Activity,    wt_Signal,      true,   false,  true,   true  },
+    { at_Activity,      wt_ObjectNode,  wt_Activity,    true,   false,  true,   true  },
+    { at_Activity,      wt_Activity,    wt_ObjectNode,  true,   false,  true,   true  },
     { at_Activity,      wt_Activity,    wt_Activity,    true,   false,  true,   true  },
     { at_Activity,      wt_ForkJoin,    wt_Activity,    true,   false,  true,   true  },
     { at_Activity,      wt_Activity,    wt_ForkJoin,    true,   false,  true,   true  },
+    { at_Activity,      wt_Signal,      wt_ForkJoin,    true,   false,  true,   true  },
+    { at_Activity,      wt_ForkJoin,    wt_Signal,      true,   false,  true,   true  },
+    { at_Activity,      wt_ForkJoin,    wt_ObjectNode,  true,   false,  true,   true  },
+    { at_Activity,      wt_ObjectNode,  wt_ForkJoin,    true,   false,  true,   true  },
+    { at_Activity,      wt_Pin,         wt_Activity,    true,   false,  true,   true  },
+    { at_Activity,      wt_Pin,         wt_Pin,         true,   false,  true,   true  },
+    { at_Activity,      wt_Activity,    wt_Pin,         true,   false,  true,   true  },
+    { at_Activity,      wt_Pin,         wt_ForkJoin,    true,   false,  true,   true  },
+    { at_Activity,      wt_ForkJoin,    wt_Pin,    true,   false,  true,   true  },
     { at_Anchor,        wt_Class,       wt_Note,        false,  false,  false,  false },
     { at_Anchor,        wt_Package,     wt_Note,        false,  false,  false,  false },
     { at_Anchor,        wt_Interface,   wt_Note,        false,  false,  false,  false },
@@ -379,7 +435,14 @@ AssocRules::Assoc_Rule AssocRules::m_AssocRules []= {
     { at_Anchor,        wt_Message,     wt_Note,        false,  false,  false,  false },
     { at_Anchor,        wt_State,       wt_Note,        false,  false,  false,  false },
     { at_Anchor,        wt_Activity,    wt_Note,        false,  false,  false,  false },
+    { at_Anchor,        wt_Entity,      wt_Note,        false,  false,  false,  false },
     { at_Relationship,  wt_Entity,      wt_Entity,      true,   true,   true,   true  },
+    { at_Exception,     wt_Activity,    wt_Activity,    true,   false,  true,   true  },
+    { at_Exception,     wt_Activity,    wt_Signal,      true,   false,  true,   true  },
+    { at_Exception,     wt_Signal,      wt_Activity,    true,   false,  true,   true  },
+    { at_Exception,     wt_Signal,      wt_Signal,      true,   false,  true,   true  },
+    { at_Category2Parent, wt_Category,  wt_Entity,      false,  false,  true,   false },
+    { at_Child2Category, wt_Entity,     wt_Category,    false,  false,  true,   false }
 };
 
 int AssocRules::m_nNumRules = sizeof( m_AssocRules ) / sizeof( AssocRules::Assoc_Rule );
