@@ -1,11 +1,10 @@
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2003-2006                                               *
+ *   copyright (C) 2003-2008                                               *
  *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
@@ -27,7 +26,8 @@
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
 
-class UMLComponent : public UMLPackage {
+class UMLComponent : public UMLPackage
+{
     Q_OBJECT
 public:
     /**
@@ -36,7 +36,7 @@ public:
      * @param name              The name of the Concept.
      * @param id                The unique id of the Concept.
      */
-    explicit UMLComponent(const QString & name = "", Uml::IDType id = Uml::id_None);
+    explicit UMLComponent(const QString & name = QString(), Uml::IDType id = Uml::id_None);
 
     /**
      * Empty deconstructor.
