@@ -13,14 +13,15 @@
 #ifndef SELECTOPDLG_H
 #define SELECTOPDLG_H
 
-#include <kdialogbase.h>
-
-#include <qgroupbox.h>
-#include <kcombobox.h>
+#include <q3groupbox.h>
 #include <qradiobutton.h>
-#include <qbuttongroup.h>
-#include <qlineedit.h>
 #include <qlabel.h>
+#include <q3buttongroup.h>
+
+#include <klineedit.h>
+#include <kcombobox.h>
+#include <kdialog.h>
+
 #include "../classifier.h"
 
 class UMLView;
@@ -36,7 +37,7 @@ class UMLView;
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
 
-class SelectOpDlg : public KDialogBase
+class SelectOpDlg : public KDialog
 {
     Q_OBJECT
 public:
@@ -102,12 +103,12 @@ protected:
     */
     void insertOperation( const QString& type, int index = -1 );
 private:
-    QGroupBox * m_pOpGB;
+    Q3GroupBox * m_pOpGB;
     KComboBox * m_pOpCB;
     QLabel * m_pSeqL;
-    QLineEdit * m_pOpLE, * m_pSeqLE;
+    KLineEdit * m_pOpLE, * m_pSeqLE;
     QRadioButton * m_pCustomRB, * m_pOpRB;
-    QButtonGroup * m_pOpBG, * m_pDocGB;
+    Q3ButtonGroup * m_pOpBG, * m_pDocGB;
     QString m_Text;
     int m_nOpCount;
     int m_id;  ///< takes on the value OP or CUSTOM according to what the user selected
