@@ -13,6 +13,9 @@
  *                                                                         *
  ***************************************************************************/
 
+// own header
+#include "pluginloader.h"
+
 // Qt includes
 #include <qstring.h>
 
@@ -21,7 +24,6 @@
 #include <klibloader.h>
 
 // u2 includes
-#include "pluginloader.h"
 #include "plugin.h"
 
 using namespace Umbrello;
@@ -62,15 +64,15 @@ PluginLoader::loadPlugin(const QString &name)
     // if the plugin has already been loaded, increment
     // its reference and return.
     if((it = _plugins.find(name)) != _plugins.end()) {
-        plugin = it.data();
+        plugin = it.value();
         plugin->ref();
         return plugin;
     }
 
     // use KLibLoader to get a reference to the library
-    lib = KLibLoader::self()->library(name.latin1());
+    lib = KLibLoader::self()->library(name);
     if(!lib) {
-        kdError() << "failed loading plugin library " << name << endl;
+        kError() << "failed loading plugin library " << name << endl;
         success = false;
     }
 
@@ -78,21 +80,22 @@ PluginLoader::loadPlugin(const QString &name)
     if(success) {
         factory = lib->factory();
         if(!factory) {
-            kdError() << "failed to find factory for " << name << endl;
+            kError() << "failed to find factory for " << name << endl;
             success = false;
         }
     }
 
     // use the factory to create the plugin
     if(success) {
-        plugin = dynamic_cast<Plugin *>(factory->create((QObject*)0, name.latin1()));
+        plugin = dynamic_cast<Plugin *>(factory->create((QObject*)0));
         if(!plugin) {
-            kdError() << "failed to create a plugin object for " << name << endl;
+            kError() << "failed to create a plugin object for " << name << endl;
             success = false;
         }
         else {
             // we have to register the plugin here, otherwise, we can get
             // recursive loads
+            plugin->setObjectName( name );
             _plugins[name] = plugin;
             _categories[plugin->category()].append(plugin);
         }
@@ -104,7 +107,7 @@ PluginLoader::loadPlugin(const QString &name)
         if(!success) {
             // on failure, delete the plugin. this should cause the
             // library to unload.
-            kdError() << "failure initializing " << name << endl;
+            kError() << "failure initializing " << name << endl;
             _categories[plugin->category()].remove(plugin);
             _plugins.remove(name);
             delete plugin;
@@ -124,18 +127,16 @@ PluginLoader::loadPlugin(const QString &name)
 Plugin *
 PluginLoader::findPlugin(const QString &name)
 {
-    Plugin *ret = NULL;
     PluginMap::iterator it = _plugins.find(name);
-    if(it != _plugins.end()) {
-        ret = it.data();
-    }
-    return ret;
+    if(it != _plugins.end())
+        return it.value();
+    return NULL;
 }
 
 void
 PluginLoader::unloadPlugin(const QString &name)
 {
-    KLibLoader::self()->unloadLibrary(name.latin1());
+    KLibLoader::self()->unloadLibrary(name);
 }
 
 const PluginLoader::PluginMap &
@@ -161,12 +162,12 @@ PluginLoader::slotDestroyed(QObject *obj)
 
     PluginMap::iterator end(_plugins.end());
     for(PluginMap::iterator i = _plugins.begin(); i != end; ++i) {
-        Plugin *p = i.data();
+        Plugin *p = i.value();
         if(p == plugin) {
-            kdDebug() << "unloading plugin " << i.key() << endl;
+            kDebug() << "unloading plugin " << i.key();
 
             // remove it from the mapping
-            _plugins.remove(i);
+            _plugins.erase(i);
             break;
         }
     }
