@@ -5,21 +5,25 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2006                                               *
+ *   copyright (C) 2002-2007                                               *
  *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 #ifndef UMLVIEW_H
 #define UMLVIEW_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 // system includes
 #include <kurl.h>
 #include <qdom.h>
-#include <qcanvas.h>
+#include <q3canvas.h>
+//Added by qt3to4:
+#include <QPixmap>
+#include <QHideEvent>
+#include <QCloseEvent>
+#include <QDropEvent>
+#include <QShowEvent>
+#include <QDragEnterEvent>
+#include <QMouseEvent>
 
 //local includes
 #include "umlobjectlist.h"
@@ -34,15 +38,18 @@ class ClassOptionsPage;
 class IDChangeLog;
 class ListPopupMenu;
 class FloatingTextWidget;
-class SeqLineWidget;
 class ObjectWidget;
 class UMLFolder;
 class UMLApp;
 class UMLDoc;
+class UMLAttribute;
 class UMLCanvasObject;
+class UMLClassifier;
 class UMLViewImageExporter;
+class UMLForeignKeyConstraint;
+class UMLEntity;
 
-class KPrinter;
+class QPrinter;
 class ToolBarState;
 class ToolBarStateFactory;
 
@@ -56,7 +63,8 @@ class ToolBarStateFactory;
  * @author Paul Hensgen <phensgen@techie.com>
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
-class UMLView : public QCanvasView {
+class UMLView : public Q3CanvasView
+{
     Q_OBJECT
 public:
     friend class UMLViewImageExporterModel;
@@ -349,7 +357,7 @@ public:
     /**
      * contains the implementation for printing functionality
      */
-    void print(KPrinter *pPrinter, QPainter & pPainter);
+    void print(QPrinter *pPrinter, QPainter & pPainter);
 
     /**
      * Overrides the standard operation.
@@ -398,10 +406,19 @@ public:
                                         UMLWidget *pWidgetA, UMLWidget *pWidgetB);
 
     /**
-     * calls findAssocWidget on three possible types:
-     * Uml::at_Aggregation << Uml::at_Composition << Uml::at_Containment
+     * Finds an association widget with the given widgets and the given role B name.
+     * Considers the following association types:
+     *  at_Association, at_UniAssociation, at_Composition, at_Aggregation
+     * This is used for seeking an attribute association.
+     *
+     * @param pWidgetA  Pointer to the UMLWidget of role A.
+     * @param pWidgetB  Pointer to the UMLWidget of role B.
+     * @param roleNameB Name at the B side of the association (the attribute name)
+     *
+     * @return Returns the widget found, returns 0 if no widget found.
      */
-    AssociationWidget * findAssocWidget(UMLWidget *pWidgetA, UMLWidget *pWidgetB);
+    AssociationWidget * findAssocWidget(UMLWidget *pWidgetA,
+                                        UMLWidget *pWidgetB, const QString& roleNameB);
 
     /**
      * Remove a widget from view.
@@ -585,6 +602,7 @@ public:
 
     /**
      * Removes a AssociationWidget from a diagram
+     * Physically deletes the AssociationWidget passed in.
      *
      * @param pAssoc  Pointer to the AssociationWidget.
      */
@@ -628,13 +646,13 @@ public:
     void showDocumentation( UMLWidget * widget, bool overwrite );
 
     /**
-    *  Calls the same method in the DocWindow.
-    */
+     *  Calls the same method in the DocWindow.
+     */
     void showDocumentation( AssociationWidget * widget, bool overwrite );
 
     /**
-    *  Calls the same method in the DocWindow.
-    */
+     *  Calls the same method in the DocWindow.
+     */
     void updateDocumentation( bool clear );
 
     /**
@@ -716,6 +734,9 @@ public:
      */
     void createAutoAttributeAssociations(UMLWidget *widget);
 
+    void createAutoConstraintAssociations(UMLWidget* widget);
+
+
     /**
      * Refreshes containment association, i.e. removes possible old
      * containment and adds new containment association if applicable.
@@ -748,7 +769,7 @@ public:
     bool showPropDialog();
 
     /**
-     * Sets some options for all the @ref ClassWidget on the view.
+     * Sets some options for all the @ref ClassifierWidget on the view.
      */
     void setClassWidgetOptions( ClassOptionsPage * page );
 
@@ -767,20 +788,6 @@ public:
      */
     bool checkUniqueSelection();
 
-    /**
-    * Adds a sequence line to the list.
-    */
-    void addSeqLine( SeqLineWidget * pLine ) {
-        m_SeqLineList.append( pLine );
-    }
-
-    /**
-    * Removes a sequence line from the list.
-    */
-    void removeSeqLine(SeqLineWidget* pLine) {
-        m_SeqLineList.remove(pLine);
-    }
-
     /**
      * Asks for confirmation and clears everything on the diagram.
      * Called from menus.
@@ -829,22 +836,22 @@ public:
     // Load/Save interface:
 
     /**
-     * Creates the <diagram> tag and fills it with the contents of the diagram.
+     * Creates the "diagram" tag and fills it with the contents of the diagram.
      */
     virtual void saveToXMI( QDomDocument & qDoc, QDomElement & qElement );
 
     /**
-     * Loads the <diagram> tag.
+     * Loads the "diagram" tag.
      */
     virtual bool loadFromXMI( QDomElement & qElement );
 
     /**
-     * Loads the <UISDiagram> tag of Unisys.IntegratePlus.2 generated files.
+     * Loads the "UISDiagram" tag of Unisys.IntegratePlus.2 generated files.
      */
     bool loadUISDiagram(QDomElement & qElement);
 
     /**
-     * Loads a <widget> element such as <UML:Class>, used by loadFromXMI() and the clipboard.
+     * Loads a "widget" element from XMI, used by loadFromXMI() and the clipboard.
      */
     UMLWidget* loadWidgetFromXMI(QDomElement& widgetElement);
 
@@ -867,6 +874,15 @@ public:
      */
     ObjectWidget * onWidgetLine( const QPoint &point );
 
+    /**
+     * Determine whether on a sequence diagram we have clicked on
+     * the destruction box of an Object.
+     *
+     * @return The widget thats destruction box was clicked on.
+     *  Returns 0 if no destruction box was clicked on.
+     */
+    ObjectWidget * onWidgetDestructionBox(const QPoint &point );
+
     /**
      * Return pointer to the first selected widget (for multi-selection)
      */
@@ -881,7 +897,7 @@ public:
      * Returns NULL if the point is not inside any widget.
      * Does not use or modify the m_pOnWidget member.
      */
-    UMLWidget *testOnWidget(QPoint p);
+    UMLWidget *getWidgetAt(const QPoint& p);
 
     /**
      * Initialize and announce a newly created widget.
@@ -915,6 +931,13 @@ public:
      */
     int generateCollaborationId();
 
+    /**
+     * Return the UMLDoc pointer
+     */
+    UMLDoc* getUMLDoc() {
+        return m_pDoc;
+    }
+
 protected:
 
     // Methods and members related to loading/saving
@@ -929,7 +952,7 @@ protected:
 
     /**
      * Contains the unique ID to allocate to a widget that needs an
-     * ID for the view.  @ref ObjectWidgets are an example of this.
+     * ID for the view.  @ref ObjectWidget is an example of this.
      */
     Uml::IDType m_nLocalID;
 
@@ -959,17 +982,17 @@ protected:
     Settings::OptionState m_Options;
 
     /**
-     * Contains all the data items for @ref MessageWidgets on the diagram.
+     * Contains all the message widgets on the diagram.
      */
     MessageWidgetList m_MessageList;
 
     /**
-     * Contains all the data for @ref UMLWidgets on the diagram.
+     * Contains all the UMLWidgets on the diagram.
      */
     UMLWidgetList m_WidgetList;
 
     /**
-     * Contains all the data for @ref AssociationWidgets on the diagram.
+     * Contains all the AssociationWidgets on the diagram.
      */
     AssociationWidgetList m_AssociationList;
 
@@ -1025,13 +1048,17 @@ protected:
     /**
      * Override standard method.
      */
-    void contentsDragEnterEvent(QDragEnterEvent* mouseEvent);
+    void dragEnterEvent(QDragEnterEvent* enterEvent);
 
     /**
      * Override standard method.
      */
-    void contentsDropEvent(QDropEvent* mouseEvent);
+    void dragMoveEvent(QDragMoveEvent* moveEvent);
 
+    /**
+     * Override standard method.
+     */
+    void dropEvent(QDropEvent* dropEvent);
 
     /**
      * Gets the smallest area to print.
@@ -1040,12 +1067,6 @@ protected:
      */
     QRect getDiagramRect();
 
-
-    /**
-     * Initializes key variables.
-     */
-    void init();
-
     /**
      * Overrides the standard operation.
      * Calls the same method in the current tool bar state.
@@ -1164,11 +1185,6 @@ private:
      */
     QPoint m_PastePoint;
 
-    /**
-     * Holds a list of all the sequence lines on a sequence diagram.
-     */
-    QPtrList<SeqLineWidget> m_SeqLineList;
-
     /**
      * Pointer to the UMLDoc
      */
@@ -1179,6 +1195,92 @@ private:
      */
     UMLViewImageExporter* m_pImageExporter;
 
+    /**
+     * Create an association with the attribute attr associated with the UMLWidget
+     * widget if the UMLClassifier type is present on the current diagram.
+     */
+    void createAutoAttributeAssociation(UMLClassifier *type,
+                                        UMLAttribute *attr,
+                                        UMLWidget *widget);
+
+    void createAutoConstraintAssociation(UMLEntity* refEntity,
+                                         UMLForeignKeyConstraint* fkConstraint,
+                                         UMLWidget* widget);
+
+    /**
+     * Returns true if the first widget's X is smaller than second's.
+     * Used for sorting the UMLWidgetList.
+     *
+     * @param widget1 The widget to compare.
+     * @param widget2 The widget to compare with.
+     */
+    static bool hasWidgetSmallerX(const UMLWidget* widget1, const UMLWidget* widget2);
+
+    /**
+     * Returns true if the first widget's Y is smaller than second's.
+     * Used for sorting the UMLWidgetList.
+     *
+     * @param widget1 The widget to compare.
+     * @param widget2 The widget to compare with.
+     */
+    static bool hasWidgetSmallerY(const UMLWidget* widget1, const UMLWidget* widget2);
+
+    /**
+     * Looks for the smallest x-value of the given UMLWidgets.
+     *
+     * @param widgetList A list with UMLWidgets.
+     */
+    int getSmallestX(const UMLWidgetList &widgetList);
+
+    /**
+     * Looks for the smallest y-value of the given UMLWidgets.
+     *
+     * @param widgetList A list with UMLWidgets.
+     */
+    int getSmallestY(const UMLWidgetList &widgetList);
+
+    /**
+     * Looks for the biggest x-value of the given UMLWidgets.
+     *
+     * @param widgetList A list with UMLWidgets.
+     */
+    int getBiggestX(const UMLWidgetList &widgetList);
+
+    /**
+     * Looks for the biggest y-value of the given UMLWidgets.
+     *
+     * @param widgetList A list with UMLWidgets.
+     */
+    int getBiggestY(const UMLWidgetList &widgetList);
+
+    /**
+     * Returns the sum of the heights of the given UMLWidgets
+     *
+     * @param widgetList A list with UMLWidgets.
+     */
+    int getHeightsSum(const UMLWidgetList &widgetList);
+
+    /**
+     * Returns the sum of the widths of the given UMLWidgets.
+     *
+     * @param widgetList A list with UMLWidgets.
+     */
+    int getWidthsSum(const UMLWidgetList &widgetList);
+
+    /**
+     * Sorts the given UMLWidgetList based on the Compare function.
+     * The list is cleared and all the widgets are added again in order.
+     *
+     * The comp function gets two const UMLWidget* params and returns
+     * a boolean telling if the first widget was smaller than the second,
+     * whatever the "smaller" concept is depending on the sorting to do.
+     *
+     * @param widgetList The list with the widgets to order.
+     * @param comp The comp function to compare the widgets.
+     */
+    template<typename Compare>
+    void sortWidgetList(UMLWidgetList &widgetList, Compare comp);
+
 public slots:
 
     void zoomIn();
@@ -1196,7 +1298,7 @@ public slots:
      * When a menu selection has been made on the menu
      * that this view created, this method gets called.
      */
-    void slotMenuSelection(int sel);
+    void slotMenuSelection(QAction* action);
 
     /**
      * This slot is entered when an event has occurred on the views display,
@@ -1231,6 +1333,46 @@ public slots:
      */
     void show();
 
+    /**
+     * Left Alignment
+     */
+    void alignLeft();
+
+    /**
+     * Right Alignment
+     */
+    void alignRight();
+
+    /**
+     * Top Alignment
+     */
+    void alignTop();
+
+    /**
+     * Bottom Alignment
+     */
+    void alignBottom();
+
+    /**
+     * Vertical Middle Alignment
+     */
+    void alignVerticalMiddle();
+
+    /**
+     * Horizontal Middle Alignment
+     */
+    void alignHorizontalMiddle();
+
+    /**
+     * Vertical Distribute Alignment
+     */
+    void alignVerticalDistribute();
+
+    /**
+     * Horizontal Distribute Alignment
+     */
+    void alignHorizontalDistribute();
+
 signals:
     void sigResetToolBar();
     void sigColorChanged( Uml::IDType );
